package sun.java2d.loops;

import sun.java2d.pipe.Region;
import sun.font.GlyphList;
import sun.java2d.SurfaceData;
import sun.java2d.SunGraphics2D;

public class DrawGlyphList extends GraphicsPrimitive
{
    public static final String methodSignature;
    public static final int primTypeID;
    
    public static DrawGlyphList locate(SurfaceType surfaceType, CompositeType compositeType, SurfaceType surfaceType2) {
        return (DrawGlyphList)GraphicsPrimitiveMgr.locate(DrawGlyphList.primTypeID, surfaceType, compositeType, surfaceType2);
    }
    
    protected DrawGlyphList(SurfaceType surfaceType, CompositeType compositeType, SurfaceType surfaceType2) {
        super(DrawGlyphList.methodSignature, DrawGlyphList.primTypeID, surfaceType, compositeType, surfaceType2);
    }
    
    public DrawGlyphList(long n, SurfaceType surfaceType, CompositeType compositeType, SurfaceType surfaceType2) {
        super(n, DrawGlyphList.methodSignature, DrawGlyphList.primTypeID, surfaceType, compositeType, surfaceType2);
    }
    
    public native void DrawGlyphList(SunGraphics2D p0, SurfaceData p1, GlyphList p2);
    
    @Override
    public GraphicsPrimitive makePrimitive(SurfaceType surfaceType, CompositeType compositeType, SurfaceType surfaceType2) {
        return new General(surfaceType, compositeType, surfaceType2);
    }
    
    @Override
    public GraphicsPrimitive traceWrap() {
        return new DrawGlyphList.TraceDrawGlyphList(this);
    }
    
    static {
        methodSignature = "DrawGlyphList(...)".toString();
        primTypeID = GraphicsPrimitive.makePrimTypeID();
        GraphicsPrimitiveMgr.registerGeneral(new DrawGlyphList(null, null, null));
    }
    
    private static class General extends DrawGlyphList
    {
        MaskFill maskop;
        
        public General(SurfaceType surfaceType, CompositeType compositeType, SurfaceType surfaceType2) {
            super(surfaceType, compositeType, surfaceType2);
            this.maskop = MaskFill.locate(surfaceType, compositeType, surfaceType2);
        }
        
        @Override
        public void DrawGlyphList(SunGraphics2D sunGraphics2D, SurfaceData surfaceData, GlyphList list) {
            list.getBounds();
            int numGlyphs = list.getNumGlyphs();
            Region compClip = sunGraphics2D.getCompClip();
            int loX = compClip.getLoX();
            int loY = compClip.getLoY();
            int hiX = compClip.getHiX();
            int hiY = compClip.getHiY();
            for (int i = 0; i < numGlyphs; ++i) {
                list.setGlyphIndex(i);
                int[] metrics = list.getMetrics();
                int n = metrics[0];
                int n2 = metrics[1];
                int n3 = metrics[2];
                int n4 = n + n3;
                int n5 = n2 + metrics[3];
                int n6 = 0;
                if (n < loX) {
                    n6 = loX - n;
                    n = loX;
                }
                if (n2 < loY) {
                    n6 += (loY - n2) * n3;
                    n2 = loY;
                }
                if (n4 > hiX) {
                    n4 = hiX;
                }
                if (n5 > hiY) {
                    n5 = hiY;
                }
                if (n4 > n && n5 > n2) {
                    this.maskop.MaskFill(sunGraphics2D, surfaceData, sunGraphics2D.composite, n, n2, n4 - n, n5 - n2, list.getGrayBits(), n6, n3);
                }
            }
        }
    }
    
    private static class TraceDrawGlyphList extends DrawGlyphList
    {
        DrawGlyphList target;
        
        public TraceDrawGlyphList(DrawGlyphList target) {
            super(target.getSourceType(), target.getCompositeType(), target.getDestType());
            this.target = target;
        }
        
        @Override
        public GraphicsPrimitive traceWrap() {
            return this;
        }
        
        @Override
        public void DrawGlyphList(SunGraphics2D sunGraphics2D, SurfaceData surfaceData, GlyphList list) {
            GraphicsPrimitive.tracePrimitive(this.target);
            this.target.DrawGlyphList(sunGraphics2D, surfaceData, list);
        }
    }
}
